import 'dart:async';

import 'package:chessboard_js/chessboard/controller.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter_hooks/flutter_hooks.dart';
import '../providers/providers.dart';
import 'package:flutter/material.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';

class NotationMoves extends HookWidget {
  NotationMoves({
    required this.controller,
    required this.notationTextColor,
    this.backgroundColor = Colors.black,
//    required this.onTapMove,
  });

  final ChessboardController controller;
  final Color backgroundColor;
  final Color notationTextColor;
//  final Function(int) onTapMove;

  @override
  Widget build(BuildContext context) {
    final moves = useProvider(moveHistoryProvider(controller)).history;

    final _scrollController = ScrollController();

    void _scrollToBottom() {
      if (_scrollController.hasClients) {
        _scrollController.animateTo(_scrollController.position.maxScrollExtent,
            duration: Duration(milliseconds: 300), curve: Curves.elasticOut);
      } else {
        Timer(Duration(milliseconds: 400), () => _scrollToBottom());
      }
    }

    final binding = WidgetsBinding.instance;
    if (binding != null) {
      WidgetsBinding.instance!.addPostFrameCallback((_) => _scrollToBottom());
    }

    return moves.isNotEmpty
        ? SingleChildScrollView(
            controller: _scrollController,
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                ...buildNotationTile(moves, Theme.of(context).dividerColor),
              ],
            ),
          )
        : const SizedBox();
  }

  /// construct the notation given the move history
  List<Widget> buildNotationTile(List<String> moves, Color dividerColor) {
    List<Widget> tiles = [];

    int count = 1;
    for (int i = 0; i < moves.length; i = i + 2) {
      final temp = Container(
        decoration: BoxDecoration(
            color: backgroundColor,
            border: Border.all(color: dividerColor),
            borderRadius: const BorderRadius.all(Radius.circular(10))),
        child: Row(
          children: [
            Container(
                decoration: BoxDecoration(
                    border:
                        Border(right: BorderSide(color: notationTextColor))),
                height: 30,
                width: 20,
                child: Center(
                    child: Text('$count',
                        style: TextStyle(color: notationTextColor)))),
            Expanded(
              flex: 2,
              child: NotationTile(
                color: notationTextColor,
                moveNumber: i,
                move: moves[i],
//                onTap: onTapMove,
              ),
            ),
            Expanded(
              flex: 2,
              child: (i + 1 < moves.length)
                  ? NotationTile(
                      color: notationTextColor,
                      move: moves[i + 1],
                      moveNumber: i + 1,
//                      onTap: onTapMove,
                    )
                  : const SizedBox(),
            ),
          ],
        ),
      );
      tiles.add(temp);
      count++;
    }
    tiles.add(const SizedBox(height: 20));
    return tiles;
  }
}

/// widget to show a complete move (white & black)
class NotationTile extends StatefulWidget {
  const NotationTile({
    Key? key,
    required this.move,
    required this.moveNumber,
    required this.color,
//    required this.onTap,
    this.tileBackgroundColor = Colors.transparent,
  }) : super(key: key);

  final String move;
  final int moveNumber;
  final Color color;
  final Color tileBackgroundColor;
//  final Function(int)? onTap;

  @override
  _NotationTileState createState() => _NotationTileState();
}

class _NotationTileState extends State<NotationTile> {
  late Color backgroundColor;

  @override
  void initState() {
    backgroundColor = widget.tileBackgroundColor;
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      height: 30,
      color: backgroundColor,
      child: GestureDetector(
//        onTap: () =>
//            widget.onTap != null ? widget.onTap!(widget.moveNumber) : null,
        child: Center(
            child: Text(widget.move, style: TextStyle(color: widget.color))),
      ),
    );
  }
}

/// widget to display player username
class BottomPlayerName extends StatelessWidget {
  const BottomPlayerName({
    required this.whiteTowardUser,
    required this.white,
    required this.black,
  });
  final bool whiteTowardUser;
  final String white;
  final String black;

  @override
  Widget build(BuildContext context) {
    return Center(
        child: Text(
      whiteTowardUser ? white : black,
      style: const TextStyle(color: Colors.white),
      overflow: TextOverflow.ellipsis,
      maxLines: 2,
    ));
  }
}

/// widget to display player username
class TopPlayerName extends StatelessWidget {
  const TopPlayerName({
    required this.whiteTowardUser,
    required this.white,
    required this.black,
  });
  final bool whiteTowardUser;
  final String white;
  final String black;

  @override
  Widget build(BuildContext context) {
    return Center(
        child: Text(
      whiteTowardUser ? black : white,
      style: const TextStyle(color: Colors.white),
      overflow: TextOverflow.ellipsis,
      maxLines: 2,
    ));
  }
}

/// container decoration use several times
class CustomContainer extends StatelessWidget {
  const CustomContainer({this.width, this.height, required this.child});

  final double? height;
  final double? width;
  final Widget child;

  @override
  Widget build(BuildContext context) {
    return Container(
      decoration: BoxDecoration(
        color: Theme.of(context).backgroundColor.withOpacity(0.7),
        border: Border.all(color: Theme.of(context).dividerColor),
        borderRadius: const BorderRadius.all(Radius.circular(10)),
      ),
      width: width,
      height: height,
      child: child,
    );
  }
}
